@@ -13,12 +13,12 @@
 #    under the License.
 
 import sys
+import urllib.parse as urlparse
 
 import glance_store as store_api
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
-import six.moves.urllib.parse as urlparse
 
 import glance.db as db_api
 from glance.i18n import _LE, _LW
@@ -70,9 +70,9 @@ def safe_delete_from_backend(context, image_id, location):
         msg = ("The image data for %(iid)s was not found in the store. "
                "The image record has been updated to reflect "
                "this." % {'iid': image_id})
-        LOG.warn(msg)
+        LOG.warning(msg)
     except store_api.StoreDeleteNotSupported as e:
-        LOG.warn(encodeutils.exception_to_unicode(e))
+        LOG.warning(encodeutils.exception_to_unicode(e))
     except store_api.UnsupportedBackend:
         exc_type = sys.exc_info()[0].__name__
         msg = (_LE('Failed to delete image %(image_id)s from store: %(exc)s') %
@@ -238,8 +238,12 @@ def _update_cinder_location_and_store_id(context, loc):
                     "due to unknown issues."), uri)
 
 
-def get_updated_store_location(locations):
+def get_updated_store_location(locations, context=None):
     for loc in locations:
+        if loc['url'].startswith("cinder://") and context:
+            _update_cinder_location_and_store_id(context, loc)
+            continue
+
         store_id = _get_store_id_from_uri(loc['url'])
         if store_id:
             loc['metadata']['store'] = store_id
